#!/bin/bash                                                                  
#
# Review phase change trigger that merges commits uploaded to Collaborator 
# by pushing to "refs/for/<target branch>" in to <target branch> when the
# Review is complete.
export PATH=/bin:$PATH

# Read input args
git="$1"
review_id="$2"                                                                  
review_phase="$3"                                                                   

# Only run when Review transitions to the "Completed" phase
if [[ $review_phase != "Completed" ]]; then
    exit 0
fi

# Set Git directory (assuming repository is bare) - note we do this again
# towards the end of the script, so if you change it here change it there
# too.
export GIT_DIR="$git"
#export GIT_DIR="$git/.git" # If non-bare repository use this instead

# Look for ref with review naming convention
review_ref="collab/review-$review_id"

# Check if Review ref exists and get commit - if ref does not exist then probably
# this Review was not created by pushing to "refs/for/<target branch>" or the 
# commit has already been merged because this Review has been through the Completed
# phase before (e.g. it was reopened by a Collaborator administrator and then 
# approved again) - ignore
commit=`git show-ref --verify --hash "refs/$review_ref"` || exit 0

# Read target branch out of ref
target_branch_ref=`git name-rev --name-only --no-undefined --refs "refs/collab/review-${review_id}-target-branch-*" "$commit"` || exit 1
target_branch=`echo "$target_branch_ref" | sed "s_^collab/review-${review_id}-target-branch-\(.*\)_\1_"` || exit 1

# Make directory to work in
merge_dir="/tmp/git-trigger-$RANDOM" #note no 'mktemp' in Windows Git
mkdir "$merge_dir"

# Report any errors by reopening the Review and making a comment
# usage: reportError <message>
reportError () {
    # sanitize input
    comment_text=`echo $1 | sed 's/[\x01-\x1F\x7F]/\n/g'`

    # Reopen the Review
    ccollab --quiet --non-interactive admin review reopen $review_id

    # Put error message in Review as a comment
    ccollab --quiet --non-interactive admin batch - <<ccollabBatchXml
        <batch-commands>
            <admin_review_comment_create>
                <review>$review_id</review>
                <comment>$comment_text</comment>
            </admin_review_comment_create>
        </batch-commands>
ccollabBatchXml

    # cleanup
    cd /tmp
    rm -rf $merge_dir

    # Exit successfully - if we exit with a non-zero error code
    # this would get logged as an error in the Collaborator log
    exit 0

}

# Create clone to do the merge
clone_output=`git clone --shared --branch "$target_branch" "$git" "$merge_dir" 2>&1` || 
    reportError "Merge-after-Review script could not clone - perhaps misconfigured or disk full? 

$clone_output"
export GIT_DIR="$merge_dir/.git"
cd $merge_dir

# Merge!
merge_output=`git merge --m "Auto-merging commit from Review $review_id" "$commit" 2>&1` ||
    reportError "Merge failed - this Review has been reopened and the Author probably needs \
to rebase their commit on to $target_branch and push to \"refs/for/$target_branch\" again \
to get the rebased commit reviewed.

Merge output:

$merge_output"

# Push to origin
push_output=`git push 2>&1` || reportError "Merge succeeded but push failed.  To retry approve this Review again.

Push output:

$push_output"

# Clean up merge dir (and cloned Git repo inside it)
cd /tmp
rm -rf $merge_dir

# Switch back to main repo
export GIT_DIR="$git"
#export GIT_DIR="$git/.git" # If non-bare repository use this instead

# Cleanup refs (note the $git/refs directory won't be cleaned up until Git runs "gc")
git update-ref -d "refs/`git name-rev --name-only --no-undefined --refs refs/collab/change-id-* $commit`"
git update-ref -d "refs/$review_ref"
git update-ref -d "refs/$target_branch_ref"


